/*
	File:		VolClus.h

	Function:	Implements volume clustering

	Author:		Andrew Willmott

    Copyright:  (c) 1997-1999, Andrew Willmott
*/

#ifndef __VolClus__
#define __VolClus__

#include "gcl/MRModel.h"

class VolClus
{
public:
	VolClus(FaceIdxArray &f, PointList &p);
	~VolClus();
	
	Void		Create();

	VolClus		*CreateHierarchy();
	Void		PushElems();
	Void		CreateBounds();

	Point		ElemCentre(Int i);
	GCLReal		ElemArea(Int i);
	Void		ElemUpdateBounds(Int i, Point &min, Point &max);

	Int			DumpHier(FILE *f, Int &id);

	Void		FindMetricStats();
	Void		CalcMetricStats(Int &n, Double &sumVol);

//	Void		DrawNodeElem(Renderer &r);
//	Void		DrawLeafElem(Renderer &r);
//	Void		DrawPatches(Renderer &r);

	Void		Print(ostream &s);

	// Fields
	Point		min, max;		// bounds
	Point		centre;
	Int			level;
	IndexList	elems;			// non-cluster elements held by this cluster
	VolClus		*child[8];		// child clusters

	GCLReal	   	clusterVol;		// volume of the cluster
	GCLReal	   	clusterArea;	// surface area contained by cluster

	FaceIdxArray	&faces;
	PointList		&points;
};

#endif
