/*
	File:		EPSPlot.h

	Purpose:	Defines vanilla class for doing 2D plotting and producing eps output.
				adopted from Paul Heckbert's libplo/ps.c
	
	Author:		Paul Heckbert, Andrew Willmott
*/

#ifndef __EPSPlot__
#define __EPSPlot__

#include "gcl/Geometry.h"
#include "gcl/Colour.h"
#include "cl/String.h"


enum { lineDashed, lineDotted, lineSolid };

class EPSPlot
{
public:
	EPSPlot();
	
	Void	Open(const Char *filename);
	Void	Close();
	
	Void	MoveTo(GCLReal x, GCLReal y);
	Void	LineTo(GCLReal x, GCLReal y);
	Void	SetWidth(GCLReal width);
	Void	LineStyle(Int style);
	Void	Dot(GCLReal x, GCLReal y, GCLReal r);
	Void	Circle(GCLReal x, GCLReal y, GCLReal r);
	Void	DrawText(GCLReal x, GCLReal y, Char *string);

	Void	SetColour(const Colour &c);
	
	Void	SetOutputSize(GCLReal size);		// measured in 1/72 of an inch.
			// Must be called before an 'Open'.
	Void	SetPort(GCLReal left, GCLReal bottom, 
				GCLReal right, GCLReal top, Bool centred);
			// If 'centred' is true, the origin will be at the centre of the port,
			// otherwise it will be placed at the bottom-left. Other positions
			// for the origin can be achieved by using 'Translate' after calling
			// SetPort.
			// If you want more than one view setup,
			// you must Push/Pop around the previous one.
			
	Void	Push();
	Void	Pop();
	Void	Translate(GCLReal tx, GCLReal ty);
	Void	Scale(GCLReal sx, GCLReal sy);
	Void	Rotate(GCLReal angle);

	Void	Comment(StrConst string);

	Bool	level2;
	// Warning: Adobe "Illustrator" can't handle level2, so it's off
	// by default.

protected:
	Void	Stroke();

	FILE	*file;
	Bool	drawn;
	GCLReal	scale;
	Char	*font;
	GCLReal	width;
	GCLReal	worldScale;
};

#endif
