/*
	File:			GLRenderer.h

	Function:		Implements an OpenGL renderer. This is still an abstract 
					object: the MakeCurrent() method must be overridden to set
					up the GL context.
	
	Author(s):		Andrew Willmott

	Copyright:		(c) 1997-2000, Andrew Willmott
 */

#ifndef __GLRenderer__
#define __GLRenderer__

#include "gcl/GCLConfig.h"
#ifndef GCL_NO_GL
#include "gcl/Renderer.h"
#include <GL/gl.h>


// --- The GLRenderer class ---------------------------------------------------


class GLRenderer : public Renderer
{
public:
	GLRenderer(Int w = 100, Int h = 100);
	
	Void				Init();
	virtual Void		Show();
	virtual Void		Print(ostream &s);
	virtual Renderer	&Begin(RenderStyle style);
	virtual Renderer	&End();

	virtual Renderer	&SetPoint(const Point &p);
	virtual Renderer	&SetNormal(const Point &p);
	virtual Renderer	&SetCoord(const Coord &c);
	virtual Renderer	&SetTexCoord(const Coord &c);
	virtual Renderer	&SetColour(const Colour &c);
	virtual Renderer	&SetColour(const Colour4 &c);

	virtual Renderer	&SetTransform(const Transform &t);
	virtual Renderer	&SetCamera(const Camera &c);
	virtual Renderer	&SetTexture(const Image *image);

	virtual Renderer	&Clear();
	virtual Renderer	&Pop();
	virtual Renderer	&Push();

	virtual Renderer	&GetImage(Image &image);
	virtual Renderer	&PutImage(const Image &image, Int x, Int y);

	// GL-specific calls

	Void				SetHeadlight(Bool on);
	Void				SetDoubleSided(Bool on);
		
protected:
	Int		glHeight;
	Int		glWidth;
	const Image *oldImage;
};

#endif
#endif
