/*
    File:       MRVertex.h

    Function:	

    Author:     Andrew Willmott, originally from code by Michael Garland,
    			but a lot's changed since then =).

    Copyright:  (c) 1997-2000, Andrew Willmott
*/

#ifndef __MRVertex__
#define __MRVertex__

#include "gcl/Geometry.h"
#include "gcl/Colour.h"
#include "cl/Flags.h"
#include "cl/NArray.h"

class MRModel;
class MRVertex;
class MRCodes;

struct FaceIdx
{
	Int		v[3];
};

typedef NArray<FaceIdx> FaceIdxArray;
typedef NArray<MRVertex> MRVertexes;
typedef NArray<MRCodes> MRCodesList;
typedef NArray<UInt32> TreeCodes;
typedef NArray<Flags8> MRFlagsList;

enum MRV_Flags
{
	MRV_Active		= 0x0001, 	///< vertex is part of current model.
	MRV_Inside		= 0x0002, 	///< vertex is inside current model.
	MRV_FaceActive	= 0x0004, 	///< face is active
	MRV_End
};

class MRVertex
{
public:
    MRVertex();

    Bool		IsLeaf()	{ return(child[0] < 0); }
    Bool		IsRoot()	{ return(parent < 0); }

	Void		SetContraction(
					MRVertexes		&vertices,
					Int				iv,
					MRFlagsList		&flags,
					Int				child0, 
					Int				child1,
					Int				face0,
					Int				face1,
					GCLReal			err
				);

    Bool		CanContract(Int iv, MRFlagsList &flags);
    Bool		CanExpand(Int iv, MRFlagsList &flags);

    Void		Contract(Int iv, MRFlagsList &flags);
    Void		Expand(Int iv, MRFlagsList &flags);

	Void		PrintID();

	Void		PullProps(MRVertexes &vertices);
				// pull face properties up the vertex tree.
	
	// fields
	
	Int			parent;			///< parent!
	Int			child[2];		///< two child vertices for an edge split
	Int			face[2];		///< faces created by this split

	Byte		level;			///< hierarchy level...	
	Vector		areaNormal;		///< average area-weighted sum normal of adj. faces
};

class MRCodes
{
public:
	MRCodes();

	Void		AdjustVertices(MRVertexes &vertices, MRFlagsList &flags,
					Int faceIndexes[3], Int leafIndexes[3]);

				// set properties of face.

	// fields
	UInt32		treeCode[3];
};


/*!
	\var treeCode

	The treeCode encodes the path from the root to a particular node m. Bit
	0 corresponds to the root node, and subsequent higher-order bits
	correspond to succesive nodes along the path to 'm'. Each bit encodes
	whether 'm' belongs to the left or right subtree of its corresponding
	node.

	The treeCode helps avoid tree traversal while refining the model. Each
	face keeps track of the treeCode of the original leaf vertex it
	contained, so that it can be used to tell which child of the current 
	vertex to select when we wish to move down the tree 'towards' that
	leaf vertex while refining the model. When we're simplifying, we can
	always just pick the parent of the current vertex.

	Of course it's not a complete solution -- if the model has more than
	32 levels (it has to be v. unbalanced for this to happen, but see
	e.g. pictureFrame.smf) we punt and simplify up from the leaves each
	time. If the contraction is balanced, the scheme can handle up to
	4 billion vertices with a 32-bit code.
 */


Void MRSetupHierarchy(
		MRVertexes		&vertices, 
		Int				iv,
		TreeCodes		&treeCodes,
		Int				&height,
		Int				level,
		Int				treeCode
	);

#endif

