/*
	File:		MatUtil.cc

	Function:	Provides useful matrix utilities

	Author:		Andrew Willmott

	Notes:		
*/

#include "gcl/MatUtil.h"

// --- image/matrix routines --------------------------------------------------

Void ImageToMat(const Image &img, Matf &mat, ImgChannel chn)
{
	Int		i;

	mat.SetSize(img.Height(), img.Width());
	
	for (i = 0; i < mat.Rows(); i++)
		img.GetRealSpan(i, 0, mat.Cols(), mat[i].Ref(), chn);
}

Void MatToImage(const Matf &mat, Image &img, ImgChannel chn)
{
	Int		i;

	if (img.Height() != mat.Rows() || img.Width() != mat.Cols())
		img.SetSize(mat.Cols(), mat.Rows());

	for (i = 0; i < mat.Rows(); i++)
		img.SetRealSpan(i, 0, mat.Cols(), mat[i].Ref(), chn);
}

Void MakeAbs(Matf &mat)
{
	Int		i, j;

	for (i = 0; i < mat.Rows(); i++)
		for (j = 0; j < mat.Cols(); j++)
			mat[i][j] = abs(mat[i][j]);
}

Void ClipToZeroOne(Matf &mat)
{
	Int		i, j;

	for (i = 0; i < mat.Rows(); i++)
		for (j = 0; j < mat.Cols(); j++)
			if (mat[i][j] < 0.0)
				mat[i][j] = 0.0;
			else if (mat[i][j] > 1.0)
				mat[i][j] = 1.0;
}

Void MakeRandomRotation(Vector x, Mat3d &M)
/*! From Jim Arvo, graphics gems II */
{
    Double  a, b, c, d, s;
    Double  z, r, theta, omega;       
    Double  bb, cc, dd;
    Double  ab, ac, ad;
    Double  bc, bd, cd;

    /* Use the random variables x[0] and x[1] to determine the axis of  */
    /* rotation in cylindrical coordinates and the random variable x[2] */
    /* to determine the amount of rotation, omega, about this axis.     */

    z = x[0];
    r = sqrt(1.0 - z * z);
    theta = 2.0 * vl_pi * x[1];
    omega = vl_pi * x[2];

    /* Compute the unit quaternion (a,b,c,d) where a is the cosine of    */
    /* half the rotation angle and the axis vector (b,c,d) is determined */
    /* by "r", "theta" and "z" computed above.                           */

    s = sin(omega);
    a = cos(omega);
    b = s * cos(theta) * r;
    c = s * sin(theta) * r;
    d = s * z;

    /* Compute all the pairwise products of a, b, c, and d, except a * a. */

    bb = b * b;   cc = c * c;   dd = d * d;
    ab = a * b;   ac = a * c;   ad = a * d;
    bc = b * c;   bd = b * d;   cd = c * d;

    /* Construct an orthogonal matrix corresponding to  */
    /* the unit quaternion (a,b,c,d).                   */

    M[0][0] = 1.0 - 2.0 * (cc + dd);
    M[0][1] =       2.0 * (bc + ad);
    M[0][2] =       2.0 * (bd - ac);

    M[1][0] =       2.0 * (bc - ad);
    M[1][1] = 1.0 - 2.0 * (bb + dd);
    M[1][2] =       2.0 * (cd + ab);

    M[2][0] =       2.0 * (bd + ac);
    M[2][1] =       2.0 * (cd - ab);
    M[2][2] = 1.0 - 2.0 * (bb + cc);
}
