/*
	File:			ScenePane.h

	Function:		'ScenePane' provides a pane which you can attach a scene
					to. The pane then lets the user manipulate the view of the
					scene.

	Author(s):		Andrew Willmott

	Copyright:		(c) 1995-2000, Andrew Willmott
 */

#ifndef __ScenePane__
#define __ScenePane__

#include "gcl/GLXPane.h"
#include "gcl/SceneObjects.h"
#include "gcl/MRModel.h"

class scFrame;

class ScenePane : public GLXPane
{
public:	
	ScenePane(Bool doubleBuf = true);
	
	virtual Void 	SetScene(scScenePtr scene);
	
	virtual Void 	Redraw();				
	// Force redraw of scene.
	Void 			ViewFrom(Point &position, Vector &normal);
	// Change viewing point
	Void			Restore();

	virtual Void	StartDrag(Coord start, Int modifiers);
	virtual Void	Drag(Coord current, Int modifiers);
	// Mouse is being dragged from start, currently at current.
	virtual Void	EndDrag(Coord end, Int modifiers);
	// Mouse was dragged from start to end.

	Void			ComplexityDrag();
	Void			SceneRotDrag();
	Void			SceneTransDrag();
			
	Void			HandleKey(Char c);
	Void			HandleExpose();
	Void			TrackMouse(Int x, Int y, Int modifiers);

	Void			SetGrid(Bool on);
	Void			SetAxes(Bool on);

	Camera*			ItsCamera() { return(itsCamera); };
	scScenePtr		ItsScene() { return(itsScene); };

	Void			SetCameraParams();
	
	MRModelsFinder	models;

protected:
	Coord			dragStart;
	Int				modifiersStart;
	Coord			offset;
	GCLReal			baseErr;

	Coord			rot;
	Vector			trans;
	GCLReal 		zoom;
	
	Coord			tRot;
	Vector			tTrans;
	GCLReal 		tZoom;
	
	Bool			wire;
	scScenePtr		itsScene;
	scFrame			*frame;
	scCamera		*itsCamera;
	Bool            gridShown;
	Bool            axesShown;
};

class ScenePane2D : public ScenePane
{
public:
	Void			SetScene(scScenePtr scene);
	Void			Drag(Coord current, Int modifiers);
};

#endif
