/*
	File:		Jacobi.cc

	Function:	

	Author:		

	Notes:		
*/

#include "Jacobi.h"
#include <iostream.h>


#define ROT(a,i,j,k,l) g=a[i][j];h=a[k][l];a[i][j]=g-s*(h+g*tau);a[k][l]=h+s*(g-h*tau);

#define MAX_ROTATIONS 100

// Description:
// Jacobi iteration for the solution of eigenvectors/eigenvalues of a 3x3
// real symmetric matrix. Square 3x3 matrix a; output eigenvalues in w;
// and output eigenvectors in v. Resulting eigenvalues/vectors are sorted
// in decreasing order; eigenvectors are normalized.
//

Bool Jacobi(const Mat3d &m, Vec3d &w, Mat3d &v)
{
    int i, j, k, iq, ip;
    double tresh, theta, tau, t, sm, s, h, g, c;
    double b[3], z[3], tmp;
    Mat3d a = m;

    // initialize
    for (ip = 0; ip < 3; ip++)
    {
        for (iq = 0; iq < 3; iq++)
        	v[ip][iq] = 0.0;

        v[ip][ip] = 1.0;
    }

    for (ip = 0; ip < 3; ip++)
    {
        b[ip] = w[ip] = a[ip][ip];
        z[ip] = 0.0;
    }

    // begin rotation sequence
    for (i = 0; i < MAX_ROTATIONS; i++)
    {
        sm = 0.0;
        for (ip = 0; ip < 2; ip++)
            for (iq = ip + 1; iq < 3; iq++)
            	sm += fabs(a[ip][iq]);

        if (sm == 0.0) 
        	break;

        if (i < 4) 
        	tresh = 0.2 * sm / (9);
        else 
        	tresh = 0.0;

        for (ip = 0; ip < 2; ip++)
        {
            for (iq = ip + 1; iq < 3; iq++)
            {
                g = 100.0 * fabs(a[ip][iq]);
                if (i > 4 && (fabs(w[ip]) + g) == fabs(w[ip])
                        && (fabs(w[iq]) + g) == fabs(w[iq]))
                {
                    a[ip][iq] = 0.0;
                }
                else if (fabs(a[ip][iq]) > tresh)
                {
                    h = w[iq] - w[ip];
                    if ( (fabs(h) + g) == fabs(h)) t = (a[ip][iq]) / h;
                    else
                    {
                        theta = 0.5 * h / (a[ip][iq]);
                        t = 1.0 / (fabs(theta) + sqrt(1.0 + theta * theta));
                        if (theta < 0.0) t = -t;
                    }
                    c = 1.0 / sqrt(1 + t * t);
                    s = t * c;
                    tau = s / (1.0 + c);
                    h = t * a[ip][iq];
                    z[ip] -= h;
                    z[iq] += h;
                    w[ip] -= h;
                    w[iq] += h;
                    a[ip][iq] = 0.0;
                    for (j = 0; j < ip - 1; j++)
                    {
                        ROT(a, j, ip, j, iq)
                    }
                    for (j = ip + 1; j < iq - 1; j++)
                    {
                        ROT(a, ip, j, j, iq)
                    }
                    for (j = iq + 1; j < 3; j++)
                    {
                        ROT(a, ip, j, iq, j)
                    }
                    for (j = 0; j < 3; j++)
                    {
                        ROT(v, j, ip, j, iq)
                    }
                }
            }
        }

        for (ip = 0; ip < 3; ip++)
        {
            b[ip] += z[ip];
            w[ip] = b[ip];
            z[ip] = 0.0;
        }
    }

    if ( i >= MAX_ROTATIONS )
    {
        cerr << "WARNING -- jacobi() -- Error computing eigenvalues." << endl;
        return false;
    }

    // sort eigenfunctions
    for (j = 0; j < 3; j++)
    {
        k = j;
        tmp = w[k];
        for (i = j; i < 3; i++)
        {
            if (w[i] >= tmp)
            {
                k = i;
                tmp = w[k];
            }
        }
        if (k != j)
        {
            w[k] = w[j];
            w[j] = tmp;
            for (i = 0; i < 3; i++)
            {
                tmp = v[i][j];
                v[i][j] = v[i][k];
                v[i][k] = tmp;
            }
        }
    }

    v = trans(v);

    return (true);
}

#undef ROT
#undef MAX_ROTATIONS

Quatf MakeQuatf(const VecTrans &R)
// R is assumed to contain the orthonormal axes of the new (rotated)
// space in its columns.
// Code originally from spiff@cs.cmu.edu
{
    GCLReal		x2, y2, w2;
	GCLReal		s;
	Quatf		q;
	
    /* Use the Matrix -> Quaternion algorithm from Shoemake's paper */

	/*
		This algorithm avoids near-zero divides by looking for a large
		component; first w, then x, y, or z. When the trace is greater
		than zero, |w| is greater than 1/2, which is as small as a
		largest component can be.  Otherwise the largest diagonal entry
		corresponds to the largest of |x|, |w| or |z|, one of which must
		be larger than |w|, and at least 1/2.
	 */

	w2 = 0.25 * (1.0 + trace(R));

    if (w2 >= 0.0)
    {
    	s = sqrt(w2);
    	q[3] = s;
    	s = 0.25 / s;
        q[0] = (R[2][1] - R[1][2]) * s;
        q[1] = (R[0][2] - R[2][0]) * s;
        q[2] = (R[1][0] - R[0][1]) * s;
    }
    else
    {
        q[3] = 0.0;
        x2 = -0.5 * (R[1][1] + R[2][2]);

        if (x2 > 0.0)
        {
        	s = sqrt(x2);
            q[0] = s;
            s = 0.5 / s;
            q[1] = R[1][0] * s;
            q[2] = R[2][0] * s;
        }
        else
        {
            q[0] = 0.0;
            y2 = 0.5 * (1.0 - R[2][2]);

            if (y2 > 0.0)
            {
            	s = sqrt(y2);
                q[1] = s;
                s = 0.5 / s;
                q[2] = R[2][1] / s;
            }
            else
            {
                q[1] = 0.0;
                q[2] = 1.0;
            }
        }
    }

	normalise(q);

	return(q);
}
