/*
	File: 		SomeScenes.cc
	
	Purpose:	Contains routines for building scenes used in the radiosity
				experiments.

	Author:		Andrew Willmott
*/


#include "SceneLang.h"
#include "Avars.h"


scScenePtr MultiBounce(Bool size)
{
	scScenePtr result;
	Int n = 4, i;
	Real a;
	Real d;

	if (size)
		a = (sqrt(5) - 1) / 4, d = (1 + 2 * a) / (3 + 4 * a);
	else
		a = 0.5;
	d = (1 + 2 * a) / (3 + 4 * a);

	Vector dx(a, 0, 0), dy(0, 0.5, 0), dz(0, 0, d);
	Point midPoint;
	Real e = 0.1;
	
	
	result = slBeginObject("multi bounce");
	slCamera();

	slAttribute(new scAvarEmittance(Avar("light", 10, 1, 1000), cWhite));
	slAttribute(new scAvarColour(Avar("reflectors", 0.5, 0.0, 1), cWhite));

	midPoint = Point(-0.25 + 0.5 * (n - 1) + 0.5, -e, -0.5);
	slBeginPoints();
	slPoint(midPoint); midPoint[0] += 2 * e;
	slPoint(midPoint); midPoint[1] += 2 * e;
	slPoint(midPoint); midPoint[0] -= 2 * e;
	slPoint(midPoint);
	slEndPoints();
	slPoly();
	
	slEmittance(cBlack);
	
	// Do rear reflectors 
	
	midPoint = Point(-0.25 - 0.5 * (n - 1), 0, -0.5);

	for (i = 0; i < n; i++)
	{
		slBeginPoints();
		slPoint(midPoint - dx - dy);
		slPoint(midPoint + dx - dy);
		slPoint(midPoint + dx + dy);
		slPoint(midPoint - dx + dy);
		slEndPoints();
		slPoly();
		midPoint[0] += 1;
	//	if (i == 0)
	//		slEmittance(cBlack);
	}
	
	// Do front reflectors
	
	midPoint = Point(0.25 - 0.5 * (n - 1), 0, 0.5);

	for (i = 0; i < n; i++)
	{
		slBeginPoints();
		slPoint(midPoint - dx - dy);
		slPoint(midPoint - dx + dy);
		slPoint(midPoint + dx + dy);
		slPoint(midPoint + dx - dy);
		slEndPoints();
		slPoly();
		midPoint[0] += 1;
	}
	
	// rear blockers
	
	slAttribute(new scAvarColour(Avar("blockers", 0.5, 0.0, 1), cWhite));

	midPoint = Point(0.25 - 0.5 * (n - 1), 0, -0.5);

	for (i = 0; i < n - 1; i++)
	{
		slBeginPoints();
		slPoint(midPoint - dy);
		slPoint(midPoint - dy + dz);
		slPoint(midPoint + dy + dz);
		slPoint(midPoint + dy);
		slEndPoints();
		slPoly();
		midPoint[0] += 1;
	}
	
	midPoint = Point(0.75 - 0.5 * (n - 1), 0, 0.5);

	for (i = 0; i < n - 1; i++)
	{
		slBeginPoints();
		slPoint(midPoint - dy);
		slPoint(midPoint + dy);
		slPoint(midPoint + dy - dz);
		slPoint(midPoint - dy - dz);
		slEndPoints();
		slPoly();
		midPoint[0] += 1;
	}
	
	slEndObject();

	return(result);
}

void SideWallPts()
// 2x2 square lying centred on the y=-1 plane, and oriented away from the origin.
{
	slBeginPoints();
	slPoint(Point(-1, -1, -1));
	slPoint(Point( 1, -1, -1));
	slPoint(Point( 1, -1,  1));
	slPoint(Point(-1, -1,  1));
	slEndPoints();
}

void Floor()
// 2x2 square lying centred on the y=0 plane, and oriented upwards.
{
	slBeginPoints();
	slPoint(Point(-1, 0,  1));
	slPoint(Point( 1, 0,  1));
	slPoint(Point( 1, 0, -1));
	slPoint(Point(-1, 0, -1));
	slEndPoints();
	slPoly();
}

void Cube()
// A 2 x 2 x 2 cube, centred around the origin, normals out.
{
	SideWallPts();
	slPoly();
	
	SideWallPts();
	slApply(Rotation(zAxis, -kHalfPi));     
	slPoly();
			
	SideWallPts();
	slApply(Rotation(zAxis, kHalfPi));      
	slPoly();
	
	SideWallPts();
	slApply(Rotation(zAxis, kPi));  
	slPoly();
	
	SideWallPts();
	slApply(Rotation(xAxis, -kHalfPi));     
	slPoly();
			
	SideWallPts();
	slApply(Rotation(xAxis, kHalfPi));      
	slPoly();
}

void Cube4()
// A 2 x 2 x 2 cube, centred around the origin, normals out, bottom & top faces removed.
{
	SideWallPts();
	slApply(Rotation(zAxis, -kHalfPi));     
	slPoly();
	
	SideWallPts();
	slApply(Rotation(zAxis, kHalfPi));      
	slPoly();
	
	SideWallPts();
	slApply(Rotation(xAxis, -kHalfPi));     
	slPoly();
			
	SideWallPts();
	slApply(Rotation(xAxis, kHalfPi));      
	slPoly();
}

void Cube5()
// A 2 x 2 x 2 cube, centred around the origin, normals out, bottom face removed.
{
	SideWallPts();
	slApply(Rotation(zAxis, kPi));     
	slPoly();
	
	SideWallPts();
	slApply(Rotation(zAxis, -kHalfPi));     
	slPoly();
	
	SideWallPts();
	slApply(Rotation(zAxis, kHalfPi));      
	slPoly();
	
	SideWallPts();
	slApply(Rotation(xAxis, -kHalfPi));     
	slPoly();
			
	SideWallPts();
	slApply(Rotation(xAxis, kHalfPi));      
	slPoly();
}

const HSVCol	clrWood(HSVCol(hsvYellow , 0.5, 0.8));
const HSVCol	clrCushion(HSVCol(hsvRed, 1, 0.5));

void Table()
{
	scObject *leg;
		
	slColour(clrCushion);

	slObject("cube");
	slApply(Scale(Vector(1, 0.05, 1)));
	slApply(Shift(Vector(0, 2.05, 0)));
	
	// It's got legs...

	slColour(clrWood);

	slObject("cube4");
	slApply(Scale(Vector(0.15, 1, 0.15)));
	slApply(Shift(Vector(0.85, 1, 0.85)));
	
	leg = slCurrent();
	
	slObject(Clone(leg));
	slApply(Rotation(yAxis, kHalfPi));
			
	slObject(Clone(leg));
	slApply(Rotation(yAxis, 2 * kHalfPi));
			
	slObject(Clone(leg));
	slApply(Rotation(yAxis, 3 * kHalfPi));
}

void Chair()
{	
	slObject("table");

	slColour(clrCushion);
	slObject("cube5");
	slApply(Scale(Vector(1, 1, 0.15)));
	slApply(Shift(Vector(0, 3.1, -0.85)));
}

void AddLibraryObjects()
{
	if (slObjectExists("cube"))
		return;

	slAddToLibrary(slBeginObject("cube"));
	Cube();
	slEndObject();
	
	slAddToLibrary(slBeginObject("cube4"));
	Cube4();
	slEndObject();
	
	slAddToLibrary(slBeginObject("cube5"));
	Cube5();
	slEndObject();
	
	slAddToLibrary(slBeginObject("wall"));
	SideWallPts();
	slPoly();
	slEndObject();
	
	slAddToLibrary(slBeginObject("floor"));
	Floor();
	slEndObject();	

	slAddToLibrary(slBeginObject("table"));
	Table();
	slEndObject();	

	slAddToLibrary(slBeginObject("chair"));
	Chair();
	slEndObject();	
}

/*
	NOTE: for the room grids, 2d (x, y) coordinate system -> (x`, y`, z`), with
	x` = x and z` = y
*/

void GridInstantiate(scScenePtr object, Int nX, Int nY, Real width, Real height)
{
	scScenePtr newObj;
	Int i, j;
	Real x, y;
	
	for (i = 0; i < nY ; i++)
		for (j = 0; j < nX; j++)
		{
			x = width * Real(2 * j - (nX - 1)) / nX;
			y = height * Real(2 * i - (nY - 1)) / nY;
			
			newObj = Clone(object);
			newObj->Apply(Shift(Vector(x, 0, y)));
			slObject(newObj);
		}
}

void ScaledGridInstantiate(scScenePtr object, Int nX, Int nY, Real width, Real height)
{
	scScenePtr newObj;
	Int i, j;
	Real x, y;
	
	for (i = 0; i < nY ; i++)
		for (j = 0; j < nX; j++)
		{
			x = width * Real(2 * j - (nX - 1)) / nX;
			y = height * Real(2 * i - (nY - 1)) / nY;
			
			newObj = Clone(object);
			newObj->Apply(Scale(Vector(1.0 / nX, 1.0, 1.0 / nY)));
			newObj->Apply(Shift(Vector(x, 0, y)));
			slObject(newObj);
		}
}

void ScaledGridInstantiate2(scScenePtr object, Int nX, Int nY, Real width, Real height)
{
	scScenePtr newObj;
	Int i, j;
	Real x, y, z = (nX + nY) / 2.0;
	
	for (i = 0; i < nY ; i++)
		for (j = 0; j < nX; j++)
		{
			x = width * Real(2 * j - (nX - 1)) / nX;
			y = height * Real(2 * i - (nY - 1)) / nY;
			
			newObj = Clone(object);
			newObj->Apply(Scale(Vector(1.0 / z, 1.0 / z, 1.0 / z)));
			newObj->Apply(Shift(Vector(x, 0, y)));
			slObject(newObj);
		}
}

void LScaledGridInstantiate(scScenePtr object, Int nX, Int nY, Real width, Real height)
{
	scScenePtr newObj;
	Int i, j;
	Real x, y;
	
	for (i = 0; i < nY - 1 ; i++)
		for (j = 0; j < nX - 1; j++)
		{
			x = width * Real(2 * j - (nX - 2)) / nX;
			y = height * Real(2 * i - (nY - 2)) / nY;
			
			newObj = Clone(object);
			newObj->Apply(Scale(Vector(1.0 / nX, 1.0, 1.0 / nY)));
			newObj->Apply(Shift(Vector(x, 0, y)));
			slObject(newObj);
		}
}

scScenePtr RComplexityScene(Int chairs)
/*
	Produces an open-plan type scene: as # chairs increases, the
	room size increases, though its height does not. A light is
	placed between each set of 4 chairs.
*/
{
	scScenePtr result;
	scScenePtr chair, light;
	Int		cx, cy, c1, c2, c3;
	Char	*sceneName = new Char[256];
	Real	lightArea;
	
	// Work out spatial parameters for the lights & chairs
		
	cx = sqrt(chairs);
	cy = cx + 1;
	c1 = Abs(sqr(cx) - chairs);
	c2 = Abs(cy * cx - chairs);
	c3 = Abs(sqr(cy) - chairs);

	if (c1 < c2)
		if (c1 < c3)
			cy = cx;
		else
			cx = cy;
	else
		if (c2 < c3)
			;
		else
			cx = cy;
	
	// Ensure we have objects to work with

	AddLibraryObjects();

	// Build light

	light = Clone((scScenePtr) slGetLibrary()->Member("floor"));
	light->Apply(Rotation(Vector(1, 0, 0), kPi));
	light->Apply(Shift(Vector(0, 2.0, 0)));
	
	// Build chair
	
	chair = Clone((scScenePtr) slGetLibrary()->Member("chair"));
	chair->Apply(Scale(BlockVec3d(0.2)));
	chair->Apply(Rotation(Vector(0, 1, 0), kHalfPi));
	chair->Apply(Scale(Vector(4.0, 1.0, 4.0)));
	
	// Start the scene
	
	sprintf(sceneName, "rcomplex_scene_%02d", chairs);
	
	result = slBeginObject(sceneName);
	slCamera();

#if 0
	if (cx > cy)
		slTransform(Scale(Vector(1.0, 4.0 / cx, Real(cy) / cx)));
	else
		slTransform(Scale(Vector(Real(cx) / cy, 4.0 / cy, 1.0)));
#else
	if (cx > cy)
		slTransform(Scale(Vector(cx, 4.0, cy)));
	else
		slTransform(Scale(Vector(cx, 4.0, cy)));

		slTransform(Scale(BlockVec3d(1 / 16.0)));
#endif
	
	// Chairs
	
	ScaledGridInstantiate(chair, cx, cy, 4.0, 4.0);	
	
	// Floor & walls
	
	slBeginObject("room");

	slColour(HSVCol(hsvBlue, 0.5, 0.51));
	slObject("floor");

	slColour(cYellow * 0.5);
	slObject("floor");			// back wall
	slApply(Rotation(Vector(1, 0, 0), kHalfPi));	
	slApply(Shift(Vector(0, 1, -1)));
	
	slObject("floor");			// right wall
	slApply(Rotation(Vector(0, 0, 1), kHalfPi));
	slApply(Shift(Vector(1, 1, 0)));	

	slObject("floor");			// left wall
	slApply(Rotation(Vector(0, 0, 1), -kHalfPi));
	slApply(Shift(Vector(-1, 1, 0)));	

	slEndObject();

	slApply(Scale(Vector(4.0, 1.0, 4.0)));
	
	// Lights
	
	slColour(cBlack);
	slAttribute(new scAvarEmittance(Avar("light_strength", 200.0, 1, 300), cWhite));
	
	LScaledGridInstantiate(light, cx, cy, 4.0, 4.0);
	slEndObject();
	
	// And now we're done.
	
	return(result);
}

scScenePtr SComplexityScene(Int chairs)
/*
	Produces an auditorium scene. As the number of chairs increase,
	the room is scaled to fit, so that the apparent height of the
	room increases.
*/
{
	scScenePtr result;
	scScenePtr chair, light;
	Int cx, cy, c1, c2, c3;
	Char *sceneName = new Char[256];
	
	// Work out spatial parameters for the lights & chairs
	
	cx = sqrt(chairs);
	cy = cx + 1;
	c1 = Abs(sqr(cx) - chairs);
	c2 = Abs(cy * cx - chairs);
	c3 = Abs(sqr(cy) - chairs);

	if (c1 < c2)
		if (c1 < c3)
			cy = cx;
		else
			cx = cy;
	else
		if (c2 < c3)
			;
		else
			cx = cy;

	// Ensure we have objects to work with

	AddLibraryObjects();

	// Build light

	light = Clone((scScenePtr) slGetLibrary()->Member("floor"));
	light->Apply(Rotation(Vector(1, 0, 0), kPi));
	light->Apply(Scale(Vector(0.5, 0.5, 0.5)));
	light->Apply(Shift(Vector(0, 2.0, 0)));
	
	// Build chair
	
	chair = Clone((scScenePtr) slGetLibrary()->Member("chair"));
	chair->Apply(Scale(BlockVec3d(0.2)));
	chair->Apply(Rotation(Vector(0, 1, 0), kHalfPi));
	chair->Apply(Scale(Vector(4.0, 2.0, 4.0)));
	
	// Start the scene
	
	sprintf(sceneName, "scomplex_scene_%02d", chairs);
	
	result = slBeginObject(sceneName);
	slCamera();
	
	slTransform(Scale((cx + cy) / 8.0 * Vector(1.0, 2.0, 1.0)));
	
	// Chairs

	ScaledGridInstantiate2(chair, cx, cy, 4.0, 4.0);
	
	// Floor & walls
	
	slBeginObject("room");

	slColour(HSVCol(hsvBlue, 0.5, 0.51));
	slObject("floor");

	slColour(cYellow * 0.5);
	slObject("floor");			// back wall
	slApply(Rotation(Vector(1, 0, 0), kHalfPi));	
	slApply(Shift(Vector(0, 1, -1)));
	
	slObject("floor");			// right wall
	slApply(Rotation(Vector(0, 0, 1), kHalfPi));
	slApply(Shift(Vector(1, 1, 0)));	

	slObject("floor");			// left wall
	slApply(Rotation(Vector(0, 0, 1), -kHalfPi));
	slApply(Shift(Vector(-1, 1, 0)));	

	slEndObject();

	slApply(Scale(Vector(4.0, 1.0, 4.0)));
	
	// Lights
	
	slColour(cBlack);
	slAttribute(new scAvarEmittance(Avar("light_strength", 130.0, 1, 300), cWhite));
	slObject(light);		
	slEndObject();
	
	// And now we're done.
	
	return(result);
}


scScenePtr TComplexityScene(Int chairs)
/*

*/
{
	scScenePtr result;
	scScenePtr chair, light;
	Int cx, cy, c1, c2, c3;
	Char *sceneName = new Char[256];
	
	// Work out spatial parameters for the lights & chairs
	
	cx = sqrt(chairs);
	cy = cx + 1;
	c1 = Abs(sqr(cx) - chairs);
	c2 = Abs(cy * cx - chairs);
	c3 = Abs(sqr(cy) - chairs);

	if (c1 < c2)
		if (c1 < c3)
			cy = cx;
		else
			cx = cy;
	else
		if (c2 < c3)
			;
		else
			cx = cy;

	// Ensure we have objects to work with

	AddLibraryObjects();

	// Build light

	light = Clone((scScenePtr) slGetLibrary()->Member("floor"));
	light->Apply(Rotation(Vector(1, 0, 0), kPi));
	light->Apply(Scale(Vector(0.5, 0.5, 0.5)));
	light->Apply(Shift(Vector(0, 2.0, 0)));
	
	// Build chair
	
	chair = Clone((scScenePtr) slGetLibrary()->Member("chair"));
	chair->Apply(Scale(BlockVec3d(0.2)));
	chair->Apply(Rotation(Vector(0, 1, 0), kHalfPi));
	chair->Apply(Scale(Vector(2.0, 1.0, 2.0) / 2.0));
	
	// Start the scene
	
	sprintf(sceneName, "tcomplex_scene_%02d", chairs);
	
	result = slBeginObject(sceneName);
	slCamera();
	
	slTransform(Scale(Vector(1.0, 2.0, 1.0)));
	
	// Chairs

	GridInstantiate(chair, cx, cy, cx * 3.0 / 5.0, cy * 3.0 / 5.0);
	
	// Floor & walls
	
	slBeginObject("room");

	slColour(HSVCol(hsvBlue, 0.5, 0.51));
	slObject("floor");

	slColour(cYellow * 0.5);
	slObject("floor");			// back wall
	slApply(Rotation(Vector(1, 0, 0), kHalfPi));	
	slApply(Shift(Vector(0, 1, -1)));
	
	slObject("floor");			// right wall
	slApply(Rotation(Vector(0, 0, 1), kHalfPi));
	slApply(Shift(Vector(1, 1, 0)));	

	slObject("floor");			// left wall
	slApply(Rotation(Vector(0, 0, 1), -kHalfPi));
	slApply(Shift(Vector(-1, 1, 0)));	

	slEndObject();

	slApply(Scale(Vector(4.0, 1.0, 4.0)));
	
	// Lights
	
	slColour(cBlack);
	slAttribute(new scAvarEmittance(Avar("light_strength", 130.0, 1, 300), cWhite));
	slObject(light);		
	slEndObject();
	
	// And now we're done.
	
	return(result);
}


scScenePtr ComplexityScene(Int chairs, Int lights)
/*
	The original complexity scene. Room size stays the same, and
	the number of chairs on the floor increases; they stay the same
	scale, but are packed closer and closer together. Ditto with
	the lights.
*/
{
	scScenePtr result;
	scScenePtr chair, light;
	Int cx, cy, c1, c2, c3;
	Int lx, ly, l1, l2, l3;
	Char *sceneName = new Char[256];
	
	// Work out spatial parameters for the lights & chairs
	
	cx = sqrt(chairs);
	cy = cx + 1;
	c1 = Abs(sqr(cx) - chairs);
	c2 = Abs(cy * cx - chairs);
	c3 = Abs(sqr(cy) - chairs);

	if (c1 < c2)
		if (c1 < c3)
			cy = cx;
		else
			cx = cy;
	else
		if (c2 < c3)
			;
		else
			cx = cy;
	
	lx = sqrt(lights);
	ly = lx + 1;
	l1 = Abs(sqr(lx) - lights);
	l2 = Abs(ly * lx - lights);
	l3 = Abs(sqr(ly) - lights);

	if (l1 < l2)
		if (l1 < l3)
			ly = lx;
		else
			lx = ly;
	else
		if (l2 < l3)
			;
		else
			lx = ly;

	// Ensure we have objects to work with

	AddLibraryObjects();

	// Build light

	light = Clone((scScenePtr) slGetLibrary()->Member("floor"));
	light->Apply(Rotation(Vector(1, 0, 0), kPi));
	light->Apply(Scale(Vector(0.25, 0.25, 0.25)));
	light->Apply(Shift(Vector(0, 2.0, 0)));
	
	// Build chair
	
	chair = Clone((scScenePtr) slGetLibrary()->Member("chair"));
	chair->Apply(Scale(BlockVec3d(0.2)));
	chair->Apply(Rotation(Vector(0, 1, 0), kHalfPi));
	
	// Start the scene
	
	sprintf(sceneName, "complex_scene_%02d_%02d", chairs, lights);
	
	result = slBeginObject(sceneName);
	slCamera();

	// Chairs
	
	GridInstantiate(chair, cx, cy, 4.0, 4.0);	
	
	// Floor & walls
	
	slBeginObject("room");

	slColour(HSVCol(hsvBlue, 0.5, 0.51));
	slObject("floor");

	slColour(cYellow * 0.5);
	slObject("floor");			// back wall
	slApply(Rotation(Vector(1, 0, 0), kHalfPi));	
	slApply(Shift(Vector(0, 1, -1)));
	
	slObject("floor");			// right wall
	slApply(Rotation(Vector(0, 0, 1), kHalfPi));
	slApply(Shift(Vector(1, 1, 0)));	

	slObject("floor");			// left wall
	slApply(Rotation(Vector(0, 0, 1), -kHalfPi));
	slApply(Shift(Vector(-1, 1, 0)));	

	slEndObject();

	slApply(Scale(Vector(4.0, 1.0, 4.0)));
	
	// Lights
	
	slColour(cBlack);
	slAttribute(new scAvarEmittance(Avar("light_strength", 200.0, 1, 300), cWhite));
	
	GridInstantiate(light, lx, ly, 3.0, 3.0);
		
	slEndObject();
	
	// And now we're done.
	
	return(result);
}


// -------------------------


scScenePtr TubeScene()
{
	scScenePtr result;
	
	// Ensure we have objects to work with

	AddLibraryObjects();

	result = slBeginObject("tube");

	slCamera();

	//	Tube
	
	slAttribute(new scAvarColour(Avar("tube_refl", 0.5, 0.0, 1.0), RGBCol(0.5, 0.5, 1.0)));

	slObject("cube4");
	slApply(Scale(Vector(-1, -4, -1)));
	
	//	Light
	
	slColour(cBlack);
	slAttribute(new scAvarEmittance(Avar("light_strength", 30, 1, 300), cWhite));

	slObject("floor");	
	slApply(Scale(Vector(0.5, 1, 0.5)));
	slApply(Shift(Vector(0, -4, 0)));
		
	slEndObject();
	
	// And now we're done.
	
	return(result);
}
