/*
	File:		Examiner.cc

	Function:	

	Author:		Andrew Willmott

	Notes:		
*/

#include "Examiner.h"
#include "gcl/SceneLang.h"
#include "gcl/MesaRenderer.h"

#include <fstream.h>


// --- Useful prims -----------------------------------------------------------

Void CentredArrow(GCLReal height, GCLReal width)
{
	const GCLReal kWidth = 0.2;
	
	slBeginObject("cArrow");
	slTransform(Scalef(width));
	
	slObject("cone");
	slApply(Shift(Vector(0, (height - width) / width, 0)));
	
	slObject("cylinder");
	slApply(Shift(0.0, 0.5, 0.0));
	slApply(Scale(Vector(0.3,  (height - width) / width, 0.3)));
	slEndObject();
}

Void Examiner::SampleDiff()
// subtract samples from the currently selected patch.
{
	Int			i, j;
	RadElem		*leaf;
	Coord		place;
	Int			density = plot->samples.Rows();
	
	if (radPane->selPatch[0])
		for (i = 0; i < density; i++)
			for (j = 0; j < density; j++)
			{
				place[0] = GCLReal(i) / GCLReal(density - 1);
				place[1] = GCLReal(j) / GCLReal(density - 1);
				
				plot->samples[i][j] -=  len((radPane->selPatch[0])->
											Sample(place));
			}
}

Void Examiner::SampleAdd()
// Add samples from the currently selected patch.
{
	Int			i, j;
	RadElem		*leaf;
	Coord		place;
	Int			density = plot->samples.Rows();
	
	if (radPane->selPatch[0])
		for (i = 0; i < density; i++)
			for (j = 0; j < density; j++)
			{
				place[0] = GCLReal(i) / GCLReal(density - 1);
				place[1] = GCLReal(j) / GCLReal(density - 1);
				
				plot->samples[i][j] +=  len((radPane->selPatch[0])->
											Sample(place));
			}
}

Void Examiner::Dump(ostream &s)
{
	Int		i, j;
	GCLReal	x;
	
	s << 0.0;
	for (j = 0; j < plot->samples.Cols(); j++)
	{
		x = j / GCLReal(plot->samples.Cols() - 1);
		s << " " << x;
	}
	s << endl;
	
	for (i = 0; i < plot->samples.Rows(); i++)
	{
		x = i / GCLReal(plot->samples.Rows() - 1);
		s << x << " ";
		for (j = 0; j < plot->samples.Cols(); j++)
			s << plot->samples[i][j] << ' ';
		s << endl;	
	}
}

Void Examiner::BuildPlot()
{
	slBeginObject("plot");

	// set up plot colour
	slColour(HSVCol(260, 1, 0.5));	
	plotColour = (scColour *) slCurrent();
	
	// set up scale transform
	slTransform(Scale(Vector(vl_1)));
	scaler = (scTransform *) slCurrent();

	// the plot!
	slObject(plot);

	slEndObject();
	
	// axes
	slColour(cWhite);
	CentredArrow(1.3, 0.1);
	slApply(Rotation(vl_x, M_PI / 2.0)); 
	CentredArrow(1.3, 0.1);
	slApply(Rotation(vl_z, -M_PI / 2.0)); 
	CentredArrow(1.3, 0.1);
	
	// base
	slPointList();
	slPoint(Point(-1, 0,  1));
	slPoint(Point( 1, 0,  1));
	slPoint(Point( 1, 0, -1));
	slPoint(Point(-1, 0, -1));

	slColour(0.5 * cWhite);
	slPoly();
	 
	slApply(Scale(Vector(1.1, 1, 1.1)));
	slApply(Shift(Vector(0, 1, 0)));
}

static double pslider_map(double val)
{
//	double d = floor(val - 0.05);
//
//	return(pow(10.0, d + 1.0) * (val - d));
	return(pow(10.0, -val));
}

static const char *psliderFilt(FL_OBJECT *, double val, int prec)
{
	static char result[32];

	val = pslider_map(val);
	sprintf(result, "%.*g", prec, val);

	return(result);
}

Void Examiner::Init(FormsGraphicsSystem &gs, RadScenePane *mainPane)
{
	plot = new scPlot2D;
	radPane = mainPane;
	
	fl_set_slider_filter(scaleSlider, psliderFilt);
	fl_set_slider_bounds(scaleSlider, -2.0, 2.0);	
	fl_set_slider_value(scaleSlider, 0.0);
	fl_set_slider_step(scaleSlider, 0.01);
   
	gs.CreateFormsPane(&plotPane, pane);
		
	plotScene = slBeginObject("plot");
	BuildPlot();
	slEndObject();
	
	plot->plotType = plotVertex;
	
	// set up camera rather than tweaking ScenePane after the
	// SetScene so that we don't get a visual jump

	scCamera *cam = new scCamera;
	cam->SetModelCentric(Coord(0.25, 0.25), vl_0);
	plotScene->Set(cam);
	plotPane.SetScene(plotScene);
	plotPane.SetHeadlight(true);
}

Void Examiner::Event(FL_OBJECT *object)
{
	if (object == dismiss)
	{
		plot->samples.SetSize(1, 1);
		Hide();
	}
	else if (object == scaleSlider)
	{
		GCLReal amount = pslider_map(fl_get_slider_value(scaleSlider));
	
		*scaler = Scale(Vector(1, amount, 1));
		plotPane.Redraw();
	}
	else if (object == diff)
	{
		SampleDiff();
		*plotColour = HSVCol(hsvRed, 0.5, 0.5);
		plotPane.Redraw();	
	}
	else if (object == add)
	{
		SampleAdd();
		*plotColour = HSVCol(hsvGreen, 0.5, 0.5);
		plotPane.Redraw();	
	}
	else if (object == dump)
	{
		StrConst	result;
		RGBAImage	image;
		FILE		*file;

		result = fl_show_fselector("Saving plot data file...", 0, 
								   "*.tif,*.tiff", "plot.tif");

		if (result)
		{
			cout << "*** dumping plot image to " << result << endl;
#if defined(GCL_MESA) && defined(RAD_MINE)
			MesaRenderer 	mr;
			RGBAImage		hqImage;
						
			image.SetSize(2 * plotPane.width, 2 * plotPane.height);
			hqImage.SetSize(plotPane.width, plotPane.height);
			mr.Init(image);
			mr.SetHeadlight(true);
			mr.SetDoubleSided(true);
			mr.SetBgColour(plotPane.GetBgColour());
			mr.Clear().Draw(plotPane.ItsScene()).Show();
			image.DownSample(hqImage);

			hqImage.Save(FileName().SetPath(result));
#else
			plotPane.MakeCurrent();
			plotPane.GetImage(image);
			image.Save(FileName().SetPath(result));
#endif
		}
	}
	else if (object == dataDump)
	{
		StrConst	result;
		ofstream	fout;
		
		result = fl_show_fselector("Saving plot data file...", 0, "*.data", 
								   "plot.data");
		
		if (result)
		{
			cout << "*** dumping plot data to " << result << endl;
			fout.open(result);
			if (fout)
			{
				Dump(fout);
				fout.close();	
			}
			else
				perror("Couldn't open output file");
		}
	}
}

