/*
	File:		FCompare.h

	Function:	float comparisons

*/

#ifndef __FCompare__
#define __FCompare__

//
// IEEE single float value tolerance equal macro by Toshi Kato ( July/28/93 )
//

#define FLOAT_EQUAL(a,b,m)  FLOAT_EQINT0(*(unsigned int *)&a,*(unsigned int *)&b,m)
#define FLOAT_EQINT0(a,b,m) ((a)<(b)?FLOAT_EQINT1(b,a,m):FLOAT_EQINT1(a,b,m))
#define FLOAT_EQINT1(a,b,m) (m>((a)-(b)))

//
// IEEE single float value tolerance equal macro ( Do not use in case of == 0.0 )
//
#define F_EQ6(a,b)  FLOAT_EQUAL(a,b,0x3f)                               //  6bit mask
#define F_EQ7(a,b)  FLOAT_EQUAL(a,b,0x7f)							    //  7bit mask
#define F_EQ8(a,b)  FLOAT_EQUAL(a,b,0xff)							    //  8bit mask
#define F_EQ9(a,b)  FLOAT_EQUAL(a,b,0x1ff)							    //  9bit mask
#define F_EQ10(a,b) FLOAT_EQUAL(a,b,0x3ff)							    // 10bit mask
#define F_EQ11(a,b) FLOAT_EQUAL(a,b,0x7ff)							    // 11bit mask
#define F_EQ12(a,b) FLOAT_EQUAL(a,b,0xfff)							    // 12bit mask
#define F_EQ13(a,b) FLOAT_EQUAL(a,b,0x1fff)							    // 13bit mask
#define F_EQ14(a,b) FLOAT_EQUAL(a,b,0x3fff)							    // 14bit mask
#define F_EQ15(a,b) FLOAT_EQUAL(a,b,0x7fff)							    // 15bit mask
#define F_EQ16(a,b) FLOAT_EQUAL(a,b,0xffff)							    // 16bit mask
#define F_EQ17(a,b) FLOAT_EQUAL(a,b,0x1ffff)						    // 17bit mask
#define F_EQ18(a,b) FLOAT_EQUAL(a,b,0x3ffff)						    // 18bit mask
#define F_EQ19(a,b) FLOAT_EQUAL(a,b,0x7ffff)						    // 19bit mask
#define F_EQ20(a,b) FLOAT_EQUAL(a,b,0xfffff)						    // 20bit mask

//
// IEEE single float value tolerance equal 1.0 macro ( i.e. == 1.0 )
//
#define F_EQ1_6(a)  FLOAT_EQINT0(*(unsigned int *)&a,0x3f800000,0x3f)   //  6bit mask
#define F_EQ1_7(a)  FLOAT_EQINT0(*(unsigned int *)&a,0x3f800000,0x7f)   //  7bit mask
#define F_EQ1_8(a)  FLOAT_EQINT0(*(unsigned int *)&a,0x3f800000,0xff)   //  8bit mask
#define F_EQ1_9(a)  FLOAT_EQINT0(*(unsigned int *)&a,0x3f800000,0x1ff)  //  9bit mask
#define F_EQ1_10(a) FLOAT_EQINT0(*(unsigned int *)&a,0x3f800000,0x3ff)  // 10bit mask

//
// IEEE single float value tolerance equal or greater than macro ( i.e. <= )
//
#define FLOAT_EQG(a,b,m)    FLOAT_EQGINT(*(unsigned int *)&a,*(unsigned int *)&b,m)
#define FLOAT_EQGINT(a,b,m) ((a)<(b)?1:FLOAT_EQINT1(a,b,m))

#define F_EQG6(a,b)  FLOAT_EQG(a,b,0x3f)                                //  6bit mask
#define F_EQG7(a,b)  FLOAT_EQG(a,b,0x7f)								//  7bit mask
#define F_EQG8(a,b)  FLOAT_EQG(a,b,0xff)								//  8bit mask
#define F_EQG9(a,b)  FLOAT_EQG(a,b,0x1ff)								//  9bit mask
#define F_EQG10(a,b) FLOAT_EQG(a,b,0x3ff)								// 10bit mask

//
// IEEE single float value tolerance equal or greater than 1.0 macro ( i.e. 1.0 <= )
//
#define F_EQG1_6(a)  FLOAT_EQGINT(0x3f800000,*(unsigned int *)&a,0x3f)  //  6bit mask
#define F_EQG1_7(a)  FLOAT_EQGINT(0x3f800000,*(unsigned int *)&a,0x7f)  //  7bit mask
#define F_EQG1_8(a)  FLOAT_EQGINT(0x3f800000,*(unsigned int *)&a,0xff)  //  8bit mask
#define F_EQG1_9(a)  FLOAT_EQGINT(0x3f800000,*(unsigned int *)&a,0x1ff) //  9bit mask
#define F_EQG1_10(a) FLOAT_EQGINT(0x3f800000,*(unsigned int *)&a,0x3ff) // 10bit mask

#endif
