/*
	File:		GenLink.cc

	Function:	

	Author:		Andrew Willmott

	Notes:		
*/

#include "GenLink.h"
#include "HRElem.h"
#include "Cluster.h"
#include "RadMethod.h"

#define DBG_COUT if (0) cerr

#define RAD_NON_ISO_SOURCE
#define RAD_NON_ISO_DEST
// the above should always be defined, except for experiments showing how
// bad the isotropic assumption is =)

// --- GenLink methods ----------------------------------------------------


Bool GenLink::CalcTransport()
{
	Assert(to->NumCoeffs() == 1, "can't handle non-haar receiver yet");	
	Assert(from->NumCoeffs() == 1, "can't handle non-haar source yet");	
	Matd T(to->NumCoeffs(), from->NumCoeffs(), &transport);

	error = to->EltCalcTransport(from, T);

	Assert(vl_is_finite(transport), "bad transport coeff!!!");

	DBG_COUT << "transport = " << T << "error = " << error << ", vis = " 
		 << visibility << endl;

#ifdef RAD_VIS
	if (gRadControl->showLinks)
	{
		Int s1 = from->eltHighlight, s2 = to->eltHighlight;
		
		from->SetHighlight(3);
		to->SetHighlight(2);

		RM_DISPLAY_START;
		gRadControl->radObject->display->Begin(renLines)
			.C(cRed)
			.P(from->EltCentre()).P(to->EltCentre())
			.End();
		RM_DISPLAY_END;

		RM_OUT1("Link transport/error: " << transport << " / " << error);
		RM_OUT2("Link Power: " << from->EltBA() << " -> " << to->EltBA());
		RM_OUT3(BFAError());
		if (RM_PAUSE) return(false);

		from->SetHighlight(s1);
		to->SetHighlight(s2);
	}
#endif

	// if there's some error, keep the link around even if it's zero
	return(transport > 0.0 || error > 0.0);
}

Bool GenLink::CalcVisibility(HRLink *parent, Bool reuse)
{
	Bool result = HRLink::CalcVisibility(parent, reuse);

	transport *= visibility;

	return(result);
}

Void GenLink::Gather()
{		
	Colour	srcB; 

	// find irradiance at (to)
	// XXX generalise
	srcB = from->B_Coeffs()[0] * transport;

	Assert(vl_is_finite(srcB[0]) && vl_is_finite(srcB[1]) && vl_is_finite(srcB[2]),
		"bad srcB coeffs");

#ifdef RAD_NON_ISO_DEST
	if (to->IsCluster() && (from != to))
	{
		// destination is a cluster: push radiosity to leaves to get 
		// proper directional distribution on the destination...

		Vector	r = from->EltCentre() - to->EltCentre();
		GCLReal	rLen = len(r);

		// r is unit vector from to -> from
		if (rLen > 0.0)
			r /= rLen;
		else
			Expect(false, "zero separation between source and destination elems.");

		if (sqrlen(srcB) > 0.0)
			to->AddIrradiance(srcB, r);
	}
	else
#endif
	{
		// XXX generalise
		to->R_Coeffs()[0] += srcB;
		DBG_COUT << "Gather: Dest(" << to->flags << ") updated to " 
			<< to->R_Coeffs()[0] << " from "
			<< from->B_Coeffs()[0] << " (" << to->flags << ")" << endl;
	}

#ifdef DEBUG
	srcB = (to->R_Coeffs()[0]);
	Assert(vl_is_finite(srcB[0]) && vl_is_finite(srcB[1]) && vl_is_finite(srcB[2]),
		"bad R coeffs");
#endif
}

GCLReal GenLink::Error()
{
	return(error);
}

GCLReal GenLink::BFAError()
{
	GCLReal		result;
	
	DBG_COUT << "BFErr: area error rho from->BA = " 
		<< to->EltArea() << ' ' 
		<< error << ' ' 
		<< to->EltRho() << ' ' 
		<< from->EltBA() << endl;

	result = to->EltArea() * error * 
		   dot(kRadRGBToLum, to->EltRho() * from->EltBA() / from->EltArea());
		   
	DBG_COUT << "BFErr: result = " << result << endl;

	return(result);
}

RefChoice GenLink::RefineOracle()
{
	// force refinement of mixed clusters, a la Gibson
	
	if (from->flags.AreSet(hrMixed))
	{
		if (from == to)
			return(kSubBoth);
		else
			return(kSubFrom);
	}
	else
		return(HRLink::RefineOracle());
}
