/*
	File:			HRMesh.h

	Function:		Defines a hierarchical mesh for use with hierarchical/
					wavelet radiosity methods.
					
	Author(s):		Andrew Willmott

	Copyright:		(c) 1997-2000, Andrew Willmott
 */


#ifndef __HRMesh__
#define __HRMesh__

#include "HRElem.h"
#include "HierMesh.h"

// --- Hierarchical Radiosity mesh element ------------------------------------


class HRMeshElem : public HRElem, public HierElem
/// An HRElem that's part of a hierarchical quad-tri mesh...
{
public:
				   	HRMeshElem();

	Void			Reset();
	Void			Print(ostream &s);
	Void			CreatePatches(PatchList &patches);

	Bool			IsLeaf()
					{ return((flags & hrIntNode) == 0); };
	Void			ApplyToChildren(Void (HRElem::*method)(Void*), 
									Void *ref = 0);
	Void			MakeChildLinks(HRElem *other, HRLink *link, 
								   Int which, Int levels);
	HRMeshElem		*Child(Int i)
					{ return((HRMeshElem*) child[i]); };

	Void			Draw(Renderer &r);
	Void			DrawNodeElem(Renderer &r);
	Void			DrawLeafElem(Renderer &r);
	Void		 	DrawMatrixRec(Void *dmi);

	Void			EltUpdateBounds(Point &min, Point &max);
	GCLReal			EltArea()	{ return(area); };
	GCLReal			EltProjArea(const Vector &v)
						   		{ return(area * Max(0.0, dot(normal, v))); };
	Point			EltCentre() { return(centre); };
	Colour			EltE()		{ return(Emittance()); };
	Colour			EltRho()	{ return(Reflectance()); };

	Void			EltSetVisPoints(HRElem *to, Point p[]);
	Void			EltGetSamples(Int numSamples, Point pts[]);

    // --- kernel sampling... -------------------------------------------------
	
	Void			SampleKernel(
							const GCLMat	&fromU,		// source patch samples			
							HRMeshElem 		*to, 		// dest. patch
							const GCLMat	&toU,		// destination samples
							GCLMat			&samples,	// results of sampling
							GCLReal 		*visPtr		// accumulate visibility
						);
					/**< Find form factors between points fromU on source and
						 points toU on the destination (to). */

	Void			SubToSubFormFactor(
							Int			nFrom, 
							HRMeshElem	*to,
							Int			nTo, 
							GCLMat		&samples, 
							GCLReal		*visPtr
						);
					/**< Form factors between nFrom x nFrom panes on this
						 patch and nTo x nTo panes on 'to'. */

	Void			EstSubFormFactor(
							Int				nFrom, 
							HRMeshElem		*to,
							const GCLMat	&toU,
							GCLMat			&samples, 
							GCLReal			*visPtr
						);
					/**< Form factors between nFrom x nFrom panes on this
						 patch and the coordinates toU on 'to'. */

	Void			DrawSampledLeaf(Renderer &r, Int n);
					///< Utility routine to draw leaf patches
	Void			HRCorrectLeaves();
					///< Call CorrectLeaves on leaf elements
	Void			SetHighlight(Int h);
};

#endif
