/*
	File:		Haar.h

	Function:	Implements haar wavelet basis over an element

	Author:		Andrew Willmott

    Copyright:  (c) 1996-2000, Andrew Willmott
*/

#ifndef __Haar__
#define __Haar__

#include "HRMesh.h"

class HaarElem;

class HaarLink : public HRLink
{
public:
	HRLink			*New() { return new HaarLink; };

	HaarElem		*To()	{ return((HaarElem*) to); };
	HaarElem		*From()	{ return((HaarElem*) from); };

	Void			Gather();	
	Bool			CalcTransport();
	Bool			CalcVisibility(HRLink *parent, Bool reuse);
	GCLReal			Error();
	GCLReal			BFAError();

	Void			Print(ostream &s);
	Void			DrawLink(Renderer &r, 
						GCLReal left, GCLReal top,
						GCLReal right, GCLReal bottom, 
					    GCLReal weight
					);

	GCLReal			Strength() { return(transport); };
	
	GCLReal 		transport;
	GCLReal 		error;
};

class HaarElem : public HRMeshElem
{
public:
	Void		SetParent(HierElem &parent);
	StrConst	Name() { return("haar"); };
	HierElem 	*New() { return(new HaarElem); };

	Colour		SampleLeaf(Coord c);
	Void		Print(ostream &s);
	Void		PrintSelf(ostream &s);
	Void		ParseSelf(istream &s);
	
	Void		DistributeColours();
	Void		DistributeColoursBest(ShadeInfo &shadeInfo);

	Void		Add();
	Void		Push();
	Void		Pull();
	Void		CalcLeafRadiosity();
	GCLReal		Error();
	Void		ClearB();
	Void		ClearR();

	HaarElem	*Child(Int i)
				{ return((HaarElem*) child[i]); };

	// gen wave stuff
	Void		EltSampleTransport(Int numSamples, Point p[], Vector n[],
						Matd &coeffs);
	GCLReal		EltCalcTransport(HRElem *from, Matd &coeffs);

	Void		AddIrradiance(const Colour &E, const Vector &m);
	Colour		GetPower(const Vector &m);

	Int			NumCoeffs() { return(1); };
	Colour		*B_Coeffs() { return(&B); };
	Colour		*R_Coeffs() { return(&R); };

	// Fields
	Colour		B;
	Colour		R;
	static Colour lastB;

};

#endif
