/*
	File:			RT_Defs.h
	
	Function:		Definitions for hierarchical grid raytracer
	
	Author(s):		Andrew Willmott

	Copyright:		(c) 1997-2000, Andrew Willmott
*/

#ifndef __RT_Defs__
#define __RT_Defs__

#include "gcl/Geometry.h"
#include "gcl/Colour.h"
#include <stdlib.h>

/*
	Compile options:
	
	RT_GRID_CACHE				cache grid hits
	RT_TRI_CACHE				cache triangle intersections

	RT_GRID_SHADE_DEBUG			set tri id to intersection level
	RT_MEM_DEFRAG				allocate small chunks of memory
	RT_SUPPORT_GENS				support general objects (not just tris)
	RT_VIS						include visualisation stuff for drawing the grid
	USE_ANSI_C					use C rather than C++ mem operations
*/

#define RT_TRI_CACHE
#define RT_GRID_CACHE

#define RT_VIS
#define USE_ANSI_C

#ifdef USE_ANSI_C
#define RT_Alloc(X) (X*) malloc(sizeof(X))
#define RT_ArrayAlloc(X, n) (X*) malloc(sizeof(X) * n)
#define RT_ArrayAllocClr(X, n) (X*) calloc(n, sizeof(X))
#define RT_Free(X) free(X)
#define RT_ArrayFree(X) free(X)
#else
#define RT_Alloc(X) (new X)
#define RT_ArrayAlloc(X, n) (new X[n])
#define RT_ArrayAllocClr(X, n) (new X[n])
#define RT_Free(X) delete (X)
#define RT_ArrayFree(X) delete[] (X)
#endif

inline ostream &operator<<(ostream &s, Int i[3])
{
	cout << '[' << i[0] << ' ' << i[1] << ' ' << i[2] << ']';
	return(s);
}

#endif
