/*
	File:		SampleView.cc

	Function:	View sample patterns generated by samplegen

	Author:		Andrew Willmott

	Notes:		
*/

#include "gcl/GLXPane.h"
#include "Samples.h"
#include "gcl/Draw.h"
#include "cl/ArgParse.h"
#include <stdlib.h>
#include <unistd.h>


static XGraphicsSystem *gXGS = 0;
static XEventPane *gPane = 0;

static Bool gStep = false;
static Int gMaxSamples = 1024;

#ifdef UNFINISHED

Void DrawSquareSamples16(Renderer &r)
{
	Int		i, j;

	r.Push();
	r.Xform(Shift(Vector(-1, -1, -1))).Xform(Scalef(2.0));
	r.C(cWhite).Begin(renPoly).P(Point(0, 0, 0)).P(Point(1, 0, 0))
		.P(Point(1, 1, 0)).P(Point(0, 1, 0)).End();

	for (j = 0; j < kNumSampArrays; j++)
	{
		Colour clr = HSVCol(360.0 * j / kNumSampArrays, 1.0, 1.0);

		r.C(clr);
		for (i = 0; i < 16; i++)
		{
			DrawCircle(r, Point(tSquareSamples16[j][i][0], 
								tSquareSamples16[j][i][1], -0.01), vl_z, 0.01);
			if (gStep)
			{
				gXGS->Run();
				gPane->HandleExpose();
			}
		}
	}

	r.Pop().Show();
}

Void DrawTriSamples16(Renderer &r)
{
	Int		i, j;

	r.Push();
	r.Xform(Shift(Vector(-1, -1, -1))).Xform(Scalef(2.0));
	r.C(cWhite).Begin(renPoly).P(Point(0, 0, 0)).P(Point(1, 0, 0))
		.P(Point(0, 1, 0)).End();

	for (j = 0; j < kNumSampArrays; j++)
	{
		Colour clr = HSVCol(360.0 * j / kNumSampArrays, 1.0, 1.0);

		r.C(clr);
		for (i = 0; i < 16; i++)
		{
			DrawCircle(r, Point(tTriangleSamples16[j][i][0], 
								tTriangleSamples16[j][i][1], -0.01),
					   vl_z, 0.01);
			if (gStep)
			{
				gXGS->Run();
				gPane->HandleExpose();
			}
		}
	}

	r.Pop().Show();
}

#endif

Void DrawCubeView(Renderer &r, const Vector &dir)
{
	Int		i;

	r.C(cWhite);
	PaintBox(r, vl_0, vl_1);

	for (i = 0; i < Min(kMaxStratSamples, gMaxSamples); i++)
	{
		Colour clr = HSVCol(360.0 * i / kMaxStratSamples, 1.0, 1.0);

		r.C(clr);

		DrawCircle(r, Point(tCubeSamples[i][0], 
			tCubeSamples[i][1], tCubeSamples[i][2]), dir, 0.01);

		if (gStep)
		{
			gXGS->Run();
			gPane->HandleExpose();
		}
	}
}

Void DrawSphereView(Renderer &r, const Vector &dir)
{
	Int		i;

	r.C(cWhite);
	scRenderSteps = 32;
	DrawCircle(r, 0.5 * (Point(vl_1) + dir), dir, 0.5);

	for (i = 0; i < Min(kMaxStratSamples, gMaxSamples); i++)
	{
		Colour clr = HSVCol(360.0 * i / kMaxStratSamples, 1.0, 1.0);

		r.C(clr);

		DrawCircle(r, 0.5 * (Point(vl_1) + Point(tSphereSamples[i][0], 
			tSphereSamples[i][1], tSphereSamples[i][2])), dir, 0.01);

		if (gStep)
		{
			gXGS->Run();
			gPane->HandleExpose();
		}
	}
}

Void Draw3DSamples(Bool sphere, Renderer &r)
{
	Camera	cam;
	
	r.Push();

#ifdef SVIEW_3D
	r.Cam(cam);
	r.Xform(Shift(Vector(-1, -1, -1))).Xform(Scalef(2.0));
	DrawCubeView(r);
#else
	r.Xform(Scalef(0.45)).Push();
	r.Xform(Shift(-2.1, 0, 0)).Xform(Rotation(vl_x, vl_halfPi));
	r.Xform(Shift(Vector(-1, -1, -1))).Xform(Scalef(2.0));
	if (sphere)
		DrawSphereView(r, vl_y);
	else
		DrawCubeView(r, vl_y);
	r.Pop().Push();
	r.Xform(Shift(Vector(-1, -1, -1))).Xform(Scalef(2.0));
	if (sphere)
		DrawSphereView(r, vl_z);
	else
		DrawCubeView(r, vl_z);
	r.Pop().Push();
	r.Xform(Shift(2.1, 0, 0)).Xform(Rotation(vl_y, -vl_halfPi));
	r.Xform(Shift(Vector(-1, -1, -1))).Xform(Scalef(2.0));
	if (sphere)
		DrawSphereView(r, vl_x);
	else
		DrawCubeView(r, vl_x);
	r.Pop();
#endif

	r.Pop().Show();
}

Void DrawSquareSamples(Renderer &r)
{
	Int		i;

	r.Push();
	r.Xform(Shift(Vector(-1, -1, -1))).Xform(Scalef(2.0));
	r.C(cWhite).Begin(renPoly).P(Point(0, 0, 0)).P(Point(1, 0, 0))
		.P(Point(1, 1, 0)).P(Point(0, 1, 0)).End();

	for (i = 0; i < Min(kMaxStratSamples, gMaxSamples); i++)
	{
		Colour clr = HSVCol(360.0 * i / kMaxStratSamples, 1.0, 1.0);

		r.C(clr);
		DrawCircle(r, Point(tSquareSamples[i][0], 
							tSquareSamples[i][1], -0.01), vl_z, 0.01);
		if (gStep)
		{
			gXGS->Run();
			gPane->HandleExpose();
		}
	}

	r.Pop().Show();
}

Void DrawTriSamples(Renderer &r)
{
	Int		i;

	r.Push();
	r.Xform(Shift(Vector(-1, -1, -1))).Xform(Scalef(2.0));
	r.C(cWhite).Begin(renPoly).P(Point(0, 0, 0)).P(Point(1, 0, 0))
		.P(Point(0, 1, 0)).End();

	for (i = 0; i < Min(kMaxStratSamples, gMaxSamples); i++)
	{
		Colour clr = HSVCol(360.0 * i / kMaxStratSamples, 1.0, 1.0);

		r.C(clr);
		DrawCircle(r, Point(tTriangleSamples[i][0], 
							tTriangleSamples[i][1], -0.01),
				   vl_z, 0.01);

		if (gStep)
		{
			gXGS->Run();
			gPane->HandleExpose();
		}
	}

	r.Pop().Show();
}

Void DrawCircleSamples(Renderer &r)
{
	Int		i;

	r.Push();

	scRenderSteps = 30;
	DrawCircle(r, vl_0, vl_z, 1.0);

	for (i = 0; i < Min(kMaxStratSamples, gMaxSamples); i++)
	{
		Colour clr = HSVCol(360.0 * i / kMaxStratSamples, 1.0, 1.0);

		r.C(clr);
		DrawCircle(r, Point(tCircleSamples[i][0], 
							tCircleSamples[i][1], -0.01),
				   vl_z, 0.02);

		if (gStep)
		{
			gXGS->Run();
			gPane->HandleExpose();
		}
	}

	r.Pop().Show();
}

Coord BoundaryPoint(Int side)
{
	if ((side & 2) == 0)
		return(Coord(drand48(), side & 1));
	else
		return(Coord(side & 1, drand48()));
}

const Int tRayC1[6] = { 0, 0, 0, 1, 1, 2 };
const Int tRayC2[6] = { 1, 2, 3, 2, 3, 3 };

const Int tStartCorners[12] = 
{ 3, 0, 2, 0, 2, 2, 
  1, 1, 1, 3, 3, 0 };
const Int tCycleLen[12] = 
{ 2, 1, 3, 3, 1, 2,
  2, 3, 1, 1, 3, 2 };

Void DrawPixel(Renderer &r)
// quick hack to draw demo pixel
{
	Int		i, numPolys = gMaxSamples, ic, j;
	Coord   corner[4];

	glDisable(GL_DEPTH_TEST);
	srand48(12);
	corner[0] = Coord(0.0, 0.0);
	corner[1] = Coord(1.0, 0.0);
	corner[2] = Coord(1.0, 1.0);
	corner[3] = Coord(0.0, 1.0);

	r.Push();
	r.Xform(Shift(Vector(-1, -1, -1))).Xform(Scalef(2.0));
	r.C(cWhite).Begin(renPoly).P(Point(0, 0, -1)).P(Point(1, 0, -1))
		.P(Point(1, 1, -1)).P(Point(0, 1, -1)).End();

	// pick random half space
	for (i = 0; i < numPolys; i++)
	{
		Colour 	clr = HSVCol((360.0 * i) / numPolys, 1.0, 1.0);
		Int		half = random() % 12, ray = half % 6;
		Coord	c1 = BoundaryPoint(tRayC1[ray]);
		Coord	c2 = BoundaryPoint(tRayC2[ray]);
		GCLReal	depth = Double(i) / numPolys;
		
		r.C(clr);

		r.Begin(renPoly);

		if (half < 6)
			r.P(c1).P(c2);
		else
			r.P(c2).P(c1);
		
		ic = tStartCorners[half];

		for (j = 0; j < tCycleLen[half]; j++)
		{
			r.P(corner[ic]);
			ic = (ic + 1) % 4;
		}	
		r.End();

		if (gStep)
		{
			gXGS->Run();
			gPane->HandleExpose();
		}
	}
	
	r.Pop().Show();
}

Int main(Int argc, Char *argv[])
{
	XGraphicsSystem 	*xgs = new XGraphicsSystem;
	GLXOffscreenPane	*pane = new GLXOffscreenPane();
	XBackedPane			*bpane = new XBackedPane;
	Int					size, noFork, tri, sq, circ, cube, sphere, step, dump;
	Double				grey, clr[3];
	Arg_form			*arg_format;

	// set options

	gXGS = xgs;

	size = 400;
	grey = 0.5;
	clr[0] = -1;

	arg_format = arg_to_form(0,
		"", 									"usage: sview [options]",

		"-square", ARG_FLAG(&sq),				"Draw square samples",
		"-triangle", ARG_FLAG(&tri),			"Draw triangle samples",
		"-circle", ARG_FLAG(&circ),				"Draw circle samples",
		"-cube", ARG_FLAG(&cube),				"Draw cube samples",
		"-sphere", ARG_FLAG(&sphere),			"Draw sphere samples",
		"-step", ARG_FLAG(&step),				"Step through samples",
		"-size %d", &size, 						"Size of window",
		"-samps %d", &gMaxSamples, 				"Number of samples to use",
		"-bgr %F", &grey,						"Set greyscale background",
		"-bgc %F %F %F", clr, clr + 1, clr + 2,	"Set colour background",
		"-dump", ARG_FLAG(&dump), 				"Dump picture",
		"-noFork", ARG_FLAG(&noFork), 			"Don't fork into background",

		"", 									"",
		"", "",

		0);

	// parse arguments
	if (arg_parse_argv(argc, argv, arg_format) < 0)
		exit(1);

#ifndef DEBUG
	// fork into background
	if (!noFork && fork())
		exit(0);
#endif

	if (step)
		gStep = true;
		
	xgs->CreateOffscreen(pane, size, size);
	if (!dump)
	{
		xgs->CreateWindow(bpane, "SampleView", size, size);
		bpane->Attach(pane);
		gPane = bpane;
	}
	
	if (clr[0] < 0)
		pane->SetBgColour(cWhite * grey);
	else
		pane->SetBgColour(Colour(clr[0], clr[1], clr[2]));

	pane->Clear();
	pane->SetTransform(Scalef(0.7));

#ifdef DRAW_PIXEL
	DrawPixel(*pane);
#else
	if (circ)
		DrawCircleSamples(*pane);
	else if (tri)
		DrawTriSamples(*pane);
	else if (cube)
		Draw3DSamples(false, *pane);
	else if (sphere)
		Draw3DSamples(true, *pane);
	else
		DrawSquareSamples(*pane);
#endif
		
	if (dump)
	{
		RGBAImage img;
		pane->GetImage(img);
		img.SaveTIFF("out.tiff");
	}
	else
		xgs->Run();
		
	return(0);
}
