/*
	File:			TestScene.cc

	Function:		See header file

	Author(s):		Andrew Willmott

	Copyright:		(c) 1997-2000, Andrew Willmott

	Notes:			

*/

#include "TestScene.h"

#include "gcl/SceneLang.h"
#include "gcl/Avars.h"
#include "gcl/Writers.h"

//#define RAD_TEX_HACK

Void SaveTestModels(FileName &name)
{
	name.SetFile("box");
	SceneWriter::Save(BoxScene(), name);
	name.SetFile("box-tetra");
	SceneWriter::Save(BoxScene(1), name);
	name.SetFile("parallel");
	SceneWriter::Save(ParallelScene(), name);
	name.SetFile("blocker");
	SceneWriter::Save(BlockerScene(), name);
	// ...
}

Int gQuadType = 0;

static Void SideWall()
{
	slPointList();
	slPoint(Point(-1, -1, -1));
	slPoint(Point( 1, -1, -1));
	slPoint(Point( 1, -1,  1));
	slPoint(Point(-1, -1,  1));
}

static Void CreateMyCube()
{
	if (slObjectExists("myCube")) return;
	slAddToLibrary(slBeginObject("myCube"));
			
	SideWall();
	slPoly();

	SideWall();
	slApply(Rotation(vl_z, -vl_halfPi));	
	slPoly();
		
	SideWall();
	slApply(Rotation(vl_z, vl_halfPi));	
	slPoly();
	
	SideWall();
	slApply(Rotation(vl_z, vl_pi));	
	slPoly();
	
	SideWall();
	slApply(Rotation(vl_x, -vl_halfPi));	
	slPoly();
		
	SideWall();
	slApply(Rotation(vl_x, vl_halfPi));	
	slPoly();
	
	slEndObject();
}

static Void Table()
{
	scPrimitive *leg;
	
	if (slObjectExists("table")) return;

	CreateMyCube();
	slAddToLibrary(slBeginObject("table"));

	slObject("myCube");
	slApply(Scale(Vector(1, 0.05, 1)));
	slApply(Shift(Vector(0, 1.95, 0)));

	leg = slObject("myCube");
	slApply(Scale(Vector(0.15, 0.95, 0.15)));
	slApply(Shift(Vector(0.85, 0.95, 0.85)));

	slObject(Clone(leg));
	slApply(Rotation(vl_y, vl_halfPi));
		
	slObject(Clone(leg));
	slApply(Rotation(vl_y, 2 * vl_halfPi));
		
	slObject(Clone(leg));
	slApply(Rotation(vl_y, 3 * vl_halfPi));
		
	slEndObject();
}

static Void CreateWall()
{
	// A 2x2 square centered in the x-z plane, at y=-1.
	if (slObjectExists("wall")) return;
	
	slAddToLibrary(slBeginObject("wall"));
	slPointList();
	slPoint(Point(-1, -1,  1));
	slPoint(Point( 1, -1,  1));
	slPoint(Point( 1, -1, -1));
	slPoint(Point(-1, -1, -1));
	if (gQuadType == 2)
		slPoint(Point(0, -1, 0));
	
	switch (gQuadType)
	{
	case 0:
		slPoly();
		break;
	case 1:
		slPointIndexes(Indexes(0, 1, 2, IDX_END));
		slPoly();
		slPointIndexes(Indexes(2, 3, 0, IDX_END));
		slPoly();
		break;
	case 2:
		slPointIndexes(Indexes(0, 1, 4, IDX_END));
	 	slPoly();
		slPointIndexes(Indexes(1, 2, 4, IDX_END));
		slPoly();
		slPointIndexes(Indexes(2, 3, 4, IDX_END));
		slPoly();
		slPointIndexes(Indexes(3, 0, 4, IDX_END));
		slPoly();
	}
	
	slEndObject();
}

scScenePtr BoxScene(Int centrepiece)
{
	scScenePtr result;
	
	CreateWall();	
	result = slBeginObject("box");
	slBeginObject("light");
	slColour(cBlack);

	slAttribute(new scAvarEmittance(Avar("light", 20, 5, 50), cWhite));
	slAttribute(new scAvarShift(Avar("height", -0.1, 0.5, -1.9), vl_y));
	slAttribute(new scAvarShift(Avar("shift", 0, -1, 1), vl_z));
	slAttribute(new scAvarRotation(Avar("rotate", 0.0, -1, 1), vl_x));
	slAttribute(new scAvarRotation(Avar("roll", 0.0, -1, 1), vl_z));
	slAttribute(new scAvarScale(Avar("size", 1, 0.1, 2), Vector(0.3, 0, 0.3)));
	slTransform(Rotation(vl_x, vl_pi));
	slObject("wall");
	slEndObject();
		
#ifdef RAD_TEX_HACK
	slCoord(Coord(0, 1));
	slCoord(Coord(0, 0));
	slCoord(Coord(1, 0));
	slCoord(Coord(1, 1));
	slTexture("$TEXTURES/grey.tif");
#endif

	slAttribute(new scAvarColour(Avar("floor", 0.8, 0.0, 1.0), cWhite));
	slObject("wall");
	
	slAttribute(new scAvarColour(Avar("rwall", 0.8, 0.0, 1.0), 
		RGBCol(1.0, 0.2, 0.2)));
	slObject("wall");
	slApply(Rotation(vl_z, -vl_halfPi));	
		
#ifdef RAD_TEX_HACK
	slTexture("$TEXTURES/curves.tif");
#endif

	slAttribute(new scAvarColour(Avar("bwall", 0.8, 0.0, 1),
		 RGBCol(0.2, 0.2, 1.0)));
	slObject("wall");
	slApply(Rotation(vl_z, vl_halfPi));	
	
	slAttribute(new scAvarColour(Avar("wwall", 0.8, 0.0, 1),
		 cWhite));
	slObject("wall");
	slApply(Rotation(vl_x, vl_halfPi));	

	if (centrepiece == 1)
	{
		slAttribute(new scAvarRotation(Avar("tetra_rotate_y", 0.0, -1, 1), vl_y));
		slAttribute(new scAvarRotation(Avar("tetra_rotate_z", 0.0, -1, 1), vl_z));
		slColour(HSVCol(200, 0.5, 1));
		slObject("tetrahedron");
		slApply(Rotation(vl_y, (35.0 / 180.0) * vl_pi));
		slApply(Shift(Vector(0, -0.5, 0)));
	}

	if (centrepiece == 2)
	{
		slBeginObject("tri");
		slColour(HSVCol(300, 0.5, 0.6));
		slAttribute(new scAvarShift(Avar("tri_height", 1, 0, 2), vl_y));
		slPointList();
		slPoint(Point(-1, -1,  1));
		slPoint(Point( 1, -1,  1));
		slPoint(Point( 1, -1, -1));
		slPoly();
		slEndObject();
		slApply(Rotation(vl_y, (35.0 / 180.0) * vl_pi));
		slApply(Scale(Vector(0.3, 1, 0.3)));
	}
		
	slEndObject();

	return(result);
}

scScenePtr ParallelScene()
// Parallel
{
	scScenePtr result;
	
	CreateWall();
	result = slBeginObject("parallel");
	slBeginObject("light");
	slColour(cBlack);

	slAttribute(new scAvarEmittance(Avar("light", 20, 5, 50), cWhite));
	slAttribute(new scAvarShift(Avar("height", 0.0, 0.0, -2.0), vl_y));
	slAttribute(new scAvarShift(Avar("shift", 0, -1, 1), vl_z));
	slAttribute(new scAvarRotation(Avar("rotate", 0.0, -1, 1), vl_x));
	slAttribute(new scAvarRotation(Avar("roll", 0.0, -1, 1), vl_z));
	slAttribute(new scAvarScale(Avar("size", 1, 0.1, 2), Vector(0.3, 0, 0.3)));
	
	slTransform(Rotation(vl_x, vl_pi));
	slObject("wall");
	slEndObject();
		
	slColour(0.5 * HSVCol(0, 0, 0.8));
	slObject("wall");
	slEndObject();

	return(result);
}

scScenePtr SidelightScene()
{
	scScenePtr result;
	
	CreateWall();
	result = slBeginObject("sidelight");
	slBeginObject("light");
	slColour(cBlack);
	
	slAttribute(new scAvarEmittance(Avar("light", 20, 5, 50), cWhite));
	slAttribute(new scAvarShift(Avar("height", -0.5, 1.5, -0.5), vl_y));
	slAttribute(new scAvarShift(Avar("shift", 0, -1, 1), vl_z));
	slAttribute(new scAvarRotation(Avar("rotate", 0.0, -1, 1), vl_x));
	slAttribute(new scAvarRotation(Avar("roll", 0.0, -1, 1), vl_z));
	slAttribute(new scAvarScale(Avar("size", 1, 0.1, 2), 
								Vector(1.0, 1.0, 0.0)));
	
	slObject("wall");
	slApply(Rotation(vl_x, vl_pi / 2));
	slApply(Scale(0.5, 0.5, 1.0));
	slEndObject();
		
#ifdef RAD_TEX_HACK
	slCoord(Coord(0, 1));
	slCoord(Coord(0, 0));
	slCoord(Coord(1, 0));
	slCoord(Coord(1, 1));
	slTexture("$TEXTURES/grey.tif");
#endif

	slColour(0.5 * HSVCol(0, 0, 0.8));
	slObject("wall");
	slEndObject();

	return(result);
}

scScenePtr BlockerScene()
{
	scScenePtr result;

	CreateWall();
	result = slBeginObject("blocker");
	slBeginObject("light");
	slColour(cBlack);

	slAttribute(new scAvarEmittance(Avar("light_strength", 20, 5, 50),
									cWhite));
	slAttribute(new scAvarShift(Avar("light_height", 0.0, 1.5, -0.9), vl_y));
	slAttribute(new scAvarShift(Avar("light_z", 0, -1, 1), vl_z));
	slAttribute(new scAvarScale(Avar("light_size", 1, 0.1, 2), 
								Vector(0.3, 0, 0.3)));
	
	slAttribute(new scAvarRotation(Avar("light_spin", 0.0, -1, 1), vl_y));
	slTransform(Rotation(vl_x, vl_pi));
	slObject("wall");
	slEndObject();
		
	slColour(HSVCol(0, 0, 0.5));
	slObject("wall");
	
	slAttribute(new scAvarRotation(Avar("blocker_spin", 0.0, -1, 1), vl_y));
	slAttribute(new scAvarShift(Avar("blocker_height", 1, 0, 2), vl_y));
	slAttribute(new scAvarScale(Avar("blocker_size", 1, 0.1, 2), 
								Vector(0.2, 0, 0.2)));

	slColour(HSVCol(0, 0.5, 0.5));
	slObject("wall");
	slEndObject();

	return(result);
}

scScenePtr AbuttingScene()
{
	scScenePtr result;

	result = slBeginObject("abutting");
	
	slBeginObject("light");
	slColour(cBlack);

	slAttribute(new scAvarEmittance(Avar("light", 20, 5, 50), cWhite));

	slAttribute(new scAvarShift(Avar("height", 0.9, 1.5, -0.9), vl_y));
	slAttribute(new scAvarShift(Avar("shift", 0, -1, 1), vl_z));
	slAttribute(new scAvarRotation(Avar("rotate", 0.0, -1, 1), vl_x));
	slAttribute(new scAvarRotation(Avar("roll", 0.0, -1, 1), vl_z));
	slAttribute(new scAvarScale(Avar("size", 1, 0.1, 2), Vector(0.3, 0, 0.3)));
	
	slTransform(Rotation(vl_x, vl_pi));
	slPointList();
	slPoint(Point(-1, 0,  1));
	slPoint(Point( 1, 0,  1));
	slPoint(Point( 1, 0, -1));
	slPoint(Point(-1, 0, -1));
	
	slPoly();
	slEndObject();
		
	slAttribute(new scAvarColour(Avar("wall_refl", 0.9, 0.0, 1), cWhite));
	slObject("wall");
	slObject("wall");
	slApply(Rotation(vl_x, vl_halfPi));	

	slEndObject();

	return(result);
}

scScenePtr TableScene()
{
	scScenePtr result;
	
	Table();
	result = slBeginObject("table_scene");
	slBeginObject("light");
	slColour(cBlack);

	slAttribute(new scAvarEmittance(Avar("lights", 20, 5, 50), cWhite));

	// three lights in a row
	slTransform(Shift(0.0, 0.0, -0.6));
	slTransform(Scale(Vector(0.2, 1.0, 0.2)));
	slTransform(Rotation(vl_x, vl_pi));
	slObject("wall");
	slObject("wall");
	slApply(Shift(Vector(-6.0, 0.0, 0.0)));
	slObject("wall");
	slApply(Shift(Vector(6.0, 0.0, 0.0)));
	slEndObject();
		
	// floor
	slColour(0.8 * cWhite);
	slObject("wall");
	slApply(Scale(2.0, 1.0, 1.0));
	
	slColour(cBlue * 0.8);
	slObject("wall");
	slApply(Rotation(vl_z, -vl_halfPi));	
	slApply(Shift(Vector(-1.0, 0.0, 0.0)));
	
	slColour(cYellow * 0.8);
	slObject("wall");
	slApply(Scale(2.0, 1.0, 1.0));
	slApply(Rotation(vl_x, vl_halfPi));	
	
	slBeginObject("scene table");
	slAttribute(new scAvarShift(Avar("shift_table", 0.8, -1, 1),
								Vector(-0.5, 0.0, -0.5)));
	slAttribute(new scAvarRotation(Avar("rotate_table", 0.2), vl_y));
	slAttribute(new scAvarScale(Avar("scale_table", 0.5), Vector(2, 0, 2)));
	slColour(HSVCol(100, 0.5, 0.5));
	slObject("table");
	slApply(Scalef(0.25));
	slApply(Shift(Vector(0, -1.0, 0)));
	slEndObject();
	
	slEndObject();

	return(result);
}

scScenePtr SimpleTriScene()
{
	scScenePtr result;

	slAddToLibrary(slBeginObject("tri"));
	slPointList();
	slPoint(Point(-1, -1,  1));
	slPoint(Point( 1, -1,  1));
	slPoint(Point( 1, -1, -1));
	slPoly();
	slEndObject();

	result = slBeginObject("simple_tri");
	
	slBeginObject("light");
	slColour(cBlack);

	slAttribute(new scAvarEmittance(Avar("light", 20, 5, 50), cWhite));

	slAttribute(new scAvarShift(Avar("height", 0.9, 1.5, -0.9), vl_y));
	slAttribute(new scAvarShift(Avar("shift", 0, -1, 1), vl_z));
	slAttribute(new scAvarRotation(Avar("rotate", 0.0, -1, 1), vl_x));
	slAttribute(new scAvarRotation(Avar("roll", 0.0, -1, 1), vl_z));
	slAttribute(new scAvarScale(Avar("size", 1, 0.1, 2), Vector(0.3, 0, 0.3)));
	
	slTransform(Rotation(vl_x, vl_pi));
	slPointList();
	slPoint(Point(-1, 0,  1));
	slPoint(Point( 1, 0,  1));
	slPoint(Point( 1, 0, -1));
	slPoint(Point(-1, 0, -1));
	
	slPoly();
	slEndObject();
		
	slColour(0.5 * HSVCol(0, 0, 0.8));
	slObject("tri");
	
	slEndObject();

	return(result);
}
